__author__ = 'Marco Bravo Mejías'
__email__ = 'marco@layfellow.net'
__version__ = '0.17.2'
__copyright__ = '2022, ' + __author__
__license__ = 'GPLv3'
__description__ = 'A command line interface for the cPanel Unrestricted API.'
__url__ = 'https://github.com/layfellow/cpanel-cli'
